/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001-2005  by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@gmx.de
 *    Borg Enders  borg@borgsoft.de
 *
 ********************************************************************/

#ifdef USE_NETWORK

#include "constants.h"

#include "server.h"
#include "connection.h"

#include "gnet/server.h"

#include "../misc/setting.h"
#include "../ui/ui.h"

#include "../utils/string.h"

Network::Server* server = NULL;

namespace Network {

  unsigned const Server::STANDARD_PORT = 2442;

  /**
   ** static constructor
   **
   ** @param	-
   **
   ** @return	new server
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  Server*
    Server::new_()
    {
      return new Network::GNet::Server();
    } // static Server* Server::new_()

  /**
   ** constructor
   **
   ** @param	-
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  Server::Server() :
    port_(STANDARD_PORT),
    is_running_(false),
    connections_()
  {
    return ;
  } // Server::Server()

  /**
   ** destructor
   **
   ** @param	-
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  Server::~Server()
  {
    this->stop();
  } // Server::~Server()

  /**
   ** starts the server
   **
   ** @param	-
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Server::start()
    {
      if (this->is_running())
	return ;

      this->is_running_ = true;
      ::ui->server_started();

      return ;
    } // void Server::start()

  /**
   ** stops the server
   **
   ** @param	-
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Server::stop()
    {
      if (!this->is_running())
	return ;

      this->is_running_ = false;

      while (!this->connections().empty())
	this->delete_connection(this->connections().back());

      ::ui->server_stopped();

      return ;
    } // void Server::stop()

  /**
   ** sets the port
   **
   ** @param	-
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Server::set_port(unsigned const port)
    {
      DEBUG_ASSERTION(!this->is_running(),
		      "Server::set_port(" << port << "):\n"
		      "  server is running");

      this->port_ = port;

      return ;
    } // void Server::set_port(unsigned port)

  /**
   ** creates a new connection
   **
   ** @param	address		address to connect to
   ** @param	port		port to connect to
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  bool
    Server::create_connection(string const& address, unsigned const port)
    {
      Connection* connection = Connection::new_(*this, address, port);
      if (connection == NULL)
	return false;

      this->new_connection(connection);

      return true;
    } // bool Server::create_connection(string address, unsigned port)

  /**
   ** -> result
   **
   ** @param	-
   **
   ** @return	the connections
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  vector<Connection*>&
    Server::connections()
    {
      return this->connections_;
    } // vector<Connection*>& Server::connections()

  /**
   ** send the text over all connections
   **
   ** @param	text	received text
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Server::send_to_all(string const& text)
    {
      for (vector<Connection*>::iterator c = this->connections().begin();
	   c != this->connections().end();
	   ++c)
	(*c)->send(text);

      return ;
    } // void Server::send_to_all(string const& text)

  /**
   ** 'player' has chatted 'message'
   **
   ** @param	player	the name of the player who has chatted
   ** @param	message	the chat message
   ** @param	self	whether this player has chattet
   **			(else the chatter comes from the network)
   **			default: true
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Server::chat(string const& player, string const& message, bool const self)
    {
      if (self) {
	//this->send_to_all("chat: " + player + ": " + message);
	this->send_to_all("chat: " + message);
      }

      return ;
    } // void Server::chat(string player, string message, bool self = true)

  /**
   ** check whether the name was changed
   **
   ** @param	type		the setting, that has changed
   ** @param	old_value	old value
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Server::setting_update(int const type, void const* const old_value)
    {
      if (type == Setting::NAME) {
	this->send_to_all("name: " + ::setting(Setting::NAME));
      }
    } // void Server::setting_update(int type, void* old_value)

  /**
   ** a new connection is added
   **
   ** @param	connection	new connection
   **				the memory is managed by this class
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Server::new_connection(Connection* const connection)
    {
      this->connections_.push_back(connection);
      ::ui->network_traffic(*connection, TRAFFIC_CONNECTION, "connection (port " + DK::Utils::String::to_string(connection->port()) + ")");
      ::ui->network_connection_new(*connection);

      connection->send_self_data();

      return ;
    } // void Server::new_connection(Connection* const connection)

  /**
   ** a connection is deleted
   **
   ** @param	connection	connection to delete
   **
   ** @return	-
   **
   ** @author	Diether Knof
   **
   ** @version	0.7.0
   **/
  void
    Server::delete_connection(Connection* const connection)
    {
      vector<Connection*>::iterator c = find(this->connections_.begin(),
					     this->connections_.end(),
					     connection);
      DEBUG_ASSERTION((c != this->connections_.end()),
		      "Server::delete_connection(connection):\n"
		      "  connection " << connection->name() << ":" << connection->port() << " not found");

      this->connections_.erase(c);
      ::ui->network_traffic(*connection, TRAFFIC_DISCONNECTION,
			    "disconnection (port " + DK::Utils::String::to_string(connection->port()) + ")");
      ::ui->network_connection_deleted(*connection);

      delete connection;

      return ;
    } // void Server::delete_connection(Connection* const connection)

} // namespace Network

#endif // #ifdef USE_NETWORK
